@@ -1,10 +1,11 @@
-from neurograd import Tensor, float32
+from neurograd import Tensor, float32, int64
 import math
 import random
 import os
 from PIL import Image
 import numpy as np
 from neurograd import xp
+from typing import Optional
 
 class Dataset:
     def __init__(self, X, y, dtype = float32):
@@ -18,10 +19,10 @@ class Dataset:
     def __iter__(self):
         for idx in range(len(self)):
             yield self[idx]
-    def shuffle(self, seed: int = 42):
+    def shuffle(self, seed: Optional[int] = None):
         indices = list(range(len(self)))
-        random.seed(seed)
-        random.shuffle(indices)
+        rng = random.Random(seed) if seed is not None else random.Random()
+        rng.shuffle(indices)
         self.X = self.X[indices]
         self.y = self.y[indices]
     def __repr__(self):
@@ -38,7 +39,7 @@ class Dataset:
 
 class DataLoader:
     def __init__(self, dataset: Dataset, batch_size: int = 32, 
-                 shuffle: bool = True, seed: int = 42):
+                 shuffle: bool = True, seed: Optional[int] = None):
         self.dataset = dataset
         self.batch_size = batch_size
         self.shuffle = shuffle
@@ -48,11 +49,7 @@ class DataLoader:
     def __getitem__(self, idx):
         start = idx * self.batch_size
         end = min((idx + 1) * self.batch_size, len(self.dataset))
-        batch = [self.dataset[i] for i in range(start, end)]  # int indices only
-        Xs, ys = zip(*batch)
-        X = Tensor(xp.stack([x.data for x in Xs], axis=0), dtype=Xs[0].dtype)
-        y = Tensor(xp.stack([y.data for y in ys], axis=0), dtype=ys[0].dtype)
-        return X, y
+        return self.dataset[start:end]
     def __iter__(self):
         if self.shuffle:
             self.dataset.shuffle(self.seed)
@@ -86,8 +83,8 @@ class ImageFolder(Dataset):
         img_normalize: bool = True,              # /255 -> float
         img_transform: callable = None,   # after numpy conversion
         target_transform: callable = None,
-        img_dtype=xp.float32,                    # handled by Tensor(...)
-        target_dtype=xp.int64,                   # handled by Tensor(...)
+        img_dtype=float32,                    # handled by Tensor(...)
+        target_dtype=int64,                   # handled by Tensor(...)
         chw: bool = True                         # return CxHxW if True, else HxWxC
     ):
         self.root = root
@@ -120,6 +117,19 @@ class ImageFolder(Dataset):
 
     def __len__(self):
         return len(self.images)
+    
+    def _apply_img_transform(self, arr: np.ndarray) -> np.ndarray:
+        if self.img_transform is None:
+            return arr
+        # Try Albumentations-style call
+        try:
+            out = self.img_transform(image=arr)
+            if isinstance(out, dict) and "image" in out:
+                return out["image"]
+        except TypeError:
+            pass
+        # Fallback: plain callable expecting ndarray
+        return self.img_transform(arr)
 
     def _load_image(self, path: str) -> np.ndarray:
         # safer open/close + deterministic convert/resize
@@ -131,17 +141,41 @@ class ImageFolder(Dataset):
             arr = np.array(img)  # uint8 HxWxC or HxW
         if arr.ndim == 2:
             arr = arr[:, :, None]
+        if self.img_transform:
+            arr = self._apply_img_transform(arr)
         if self.chw:
             arr = np.transpose(arr, (2, 0, 1))  # C,H,W
         if self.img_normalize:
             arr = arr.astype(np.float32) / 255.0
-        if self.img_transform:
-            arr = self.img_transform(arr)
         return arr
 
-    def __getitem__(self, idx: int):
-        img_path = self.images[idx]
-        target_name = self.targets[idx]
+    def __getitem__(self, idx):
+        # Support single index, slices, and index sequences for batching
+        if isinstance(idx, slice) or isinstance(idx, (list, tuple, np.ndarray)):
+            if isinstance(idx, slice):
+                indices = list(range(*idx.indices(len(self))))
+            else:
+                indices = list(map(int, idx))
+
+            images = []
+            targets = []
+            for i in indices:
+                img_path = self.images[i]
+                target_name = self.targets[i]
+                img = self._load_image(img_path)
+                tgt = self.target_mapping[target_name]
+                if self.target_transform:
+                    tgt = self.target_transform(tgt)
+                images.append(img)
+                targets.append(tgt)
+
+            X = Tensor(xp.stack(images, axis=0), dtype=self.img_dtype)
+            y = Tensor(xp.asarray(targets), dtype=self.target_dtype)
+            return X, y
+
+        # Single index path
+        img_path = self.images[int(idx)]
+        target_name = self.targets[int(idx)]
 
         image = self._load_image(img_path)
         target = self.target_mapping[target_name]
@@ -152,8 +186,8 @@ class ImageFolder(Dataset):
         # Convert to Tensor with dtype specified in init
         return Tensor(image, dtype=self.img_dtype), Tensor(target, dtype=self.target_dtype)
 
-    def shuffle(self, seed: int = 42):
-        rng = random.Random(seed)
+    def shuffle(self, seed: Optional[int] = None):
+        rng = random.Random(seed) if seed is not None else random.Random()
         idxs = list(range(len(self)))
         rng.shuffle(idxs)
         self.images = [self.images[i] for i in idxs]
